/****************************************************************************
** Meta object code from reading C++ file 'dconfigserver.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../dconfigserver.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dconfigserver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15DSGConfigServerE_t {};
} // unnamed namespace

template <> constexpr inline auto DSGConfigServer::qt_create_metaobjectdata<qt_meta_tag_ZN15DSGConfigServerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DSGConfigServer",
        "releaseResource",
        "",
        "ConnKey",
        "resource",
        "tryExit",
        "acquireManager",
        "QDBusObjectPath",
        "appid",
        "name",
        "subpath",
        "acquireManagerV2",
        "uid",
        "update",
        "path",
        "sync",
        "setDelayReleaseTime",
        "ms",
        "delayReleaseTime",
        "enableVerboseLogging",
        "disableVerboseLogging",
        "setLogRules",
        "rules",
        "removeUserData",
        "reload",
        "onReleaseChanged",
        "ConnServiceName",
        "service",
        "connKey",
        "addConnWatchedService",
        "onReleaseResource",
        "onTryExit",
        "doSyncConfigCache",
        "ConfigSyncBatchRequest",
        "request"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'releaseResource'
        QtMocHelpers::SignalData<void(const ConnKey &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'tryExit'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'acquireManager'
        QtMocHelpers::SlotData<QDBusObjectPath(const QString &, const QString &, const QString &)>(6, 2, QMC::AccessPublic, 0x80000000 | 7, {{
            { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'acquireManagerV2'
        QtMocHelpers::SlotData<QDBusObjectPath(const uint &, const QString &, const QString &, const QString &)>(11, 2, QMC::AccessPublic, 0x80000000 | 7, {{
            { QMetaType::UInt, 12 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'update'
        QtMocHelpers::SlotData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'sync'
        QtMocHelpers::SlotData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'setDelayReleaseTime'
        QtMocHelpers::SlotData<void(const int)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 17 },
        }}),
        // Slot 'delayReleaseTime'
        QtMocHelpers::SlotData<int() const>(18, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'enableVerboseLogging'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'disableVerboseLogging'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setLogRules'
        QtMocHelpers::SlotData<void(const QString &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 22 },
        }}),
        // Slot 'removeUserData'
        QtMocHelpers::SlotData<void(const uint &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 12 },
        }}),
        // Slot 'reload'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onReleaseChanged'
        QtMocHelpers::SlotData<void(const ConnServiceName &, const ConnKey &)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 26, 27 }, { 0x80000000 | 3, 28 },
        }}),
        // Slot 'addConnWatchedService'
        QtMocHelpers::SlotData<void(const ConnServiceName &)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Slot 'onReleaseResource'
        QtMocHelpers::SlotData<void(const ConnKey &)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 28 },
        }}),
        // Slot 'onTryExit'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'doSyncConfigCache'
        QtMocHelpers::SlotData<void(const ConfigSyncBatchRequest &)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DSGConfigServer, qt_meta_tag_ZN15DSGConfigServerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DSGConfigServer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DSGConfigServerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DSGConfigServerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15DSGConfigServerE_t>.metaTypes,
    nullptr
} };

void DSGConfigServer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DSGConfigServer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->releaseResource((*reinterpret_cast<std::add_pointer_t<ConnKey>>(_a[1]))); break;
        case 1: _t->tryExit(); break;
        case 2: { QDBusObjectPath _r = _t->acquireManager((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusObjectPath*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusObjectPath _r = _t->acquireManagerV2((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QDBusObjectPath*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->update((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->sync((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->setDelayReleaseTime((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: { int _r = _t->delayReleaseTime();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->enableVerboseLogging(); break;
        case 9: _t->disableVerboseLogging(); break;
        case 10: _t->setLogRules((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->removeUserData((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 12: _t->reload(); break;
        case 13: _t->onReleaseChanged((*reinterpret_cast<std::add_pointer_t<ConnServiceName>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ConnKey>>(_a[2]))); break;
        case 14: _t->addConnWatchedService((*reinterpret_cast<std::add_pointer_t<ConnServiceName>>(_a[1]))); break;
        case 15: _t->onReleaseResource((*reinterpret_cast<std::add_pointer_t<ConnKey>>(_a[1]))); break;
        case 16: _t->onTryExit(); break;
        case 17: _t->doSyncConfigCache((*reinterpret_cast<std::add_pointer_t<ConfigSyncBatchRequest>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DSGConfigServer::*)(const ConnKey & )>(_a, &DSGConfigServer::releaseResource, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DSGConfigServer::*)()>(_a, &DSGConfigServer::tryExit, 1))
            return;
    }
}

const QMetaObject *DSGConfigServer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DSGConfigServer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DSGConfigServerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int DSGConfigServer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void DSGConfigServer::releaseResource(const ConnKey & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DSGConfigServer::tryExit()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
