/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.desktopspec.ConfigManager.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "configmanager_adaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class DSGConfigAdaptor
 */

DSGConfigAdaptor::DSGConfigAdaptor(DSGConfigServer *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

DSGConfigAdaptor::~DSGConfigAdaptor()
{
    // destructor
}

QDBusObjectPath DSGConfigAdaptor::acquireManager(const QString &appid, const QString &name, const QString &subpath)
{
    // handle method call org.desktopspec.ConfigManager.acquireManager
    return parent()->acquireManager(appid, name, subpath);
}

QDBusObjectPath DSGConfigAdaptor::acquireManagerV2(uint uid, const QString &appid, const QString &name, const QString &subpath)
{
    // handle method call org.desktopspec.ConfigManager.acquireManagerV2
    return parent()->acquireManagerV2(uid, appid, name, subpath);
}

int DSGConfigAdaptor::delayReleaseTime()
{
    // handle method call org.desktopspec.ConfigManager.delayReleaseTime
    return parent()->delayReleaseTime();
}

void DSGConfigAdaptor::disableVerboseLogging()
{
    // handle method call org.desktopspec.ConfigManager.disableVerboseLogging
    parent()->disableVerboseLogging();
}

void DSGConfigAdaptor::enableVerboseLogging()
{
    // handle method call org.desktopspec.ConfigManager.enableVerboseLogging
    parent()->enableVerboseLogging();
}

void DSGConfigAdaptor::reload()
{
    // handle method call org.desktopspec.ConfigManager.reload
    parent()->reload();
}

void DSGConfigAdaptor::removeUserData(uint uid)
{
    // handle method call org.desktopspec.ConfigManager.removeUserData
    parent()->removeUserData(uid);
}

void DSGConfigAdaptor::setDelayReleaseTime(int time)
{
    // handle method call org.desktopspec.ConfigManager.setDelayReleaseTime
    parent()->setDelayReleaseTime(time);
}

void DSGConfigAdaptor::setLogRules(const QString &rules)
{
    // handle method call org.desktopspec.ConfigManager.setLogRules
    parent()->setLogRules(rules);
}

void DSGConfigAdaptor::sync(const QString &path)
{
    // handle method call org.desktopspec.ConfigManager.sync
    parent()->sync(path);
}

void DSGConfigAdaptor::update(const QString &path)
{
    // handle method call org.desktopspec.ConfigManager.update
    parent()->update(path);
}


#include "moc_configmanager_adaptor.cpp"
