/****************************************************************************
** Meta object code from reading C++ file 'configmanager_adaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "configmanager_adaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'configmanager_adaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16DSGConfigAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto DSGConfigAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN16DSGConfigAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DSGConfigAdaptor",
        "D-Bus Interface",
        "org.desktopspec.ConfigManager",
        "D-Bus Introspection",
        "  <interface name=\"org.desktopspec.ConfigManager\">\n    <method "
        "name=\"acquireManagerV2\">\n      <arg direction=\"in\" type=\"u\""
        " name=\"uid\"/>\n      <arg direction=\"in\" type=\"s\" name=\"app"
        "id\"/>\n      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n  "
        "    <arg direction=\"in\" type=\"s\" name=\"subpath\"/>\n      <ar"
        "g direction=\"out\" type=\"o\" name=\"path\"/>\n    </method>\n   "
        " <method name=\"acquireManager\">\n      <arg direction=\"in\" typ"
        "e=\"s\" name=\"appid\"/>\n      <arg direction=\"in\" type=\"s\" n"
        "ame=\"name\"/>\n      <arg direction=\"in\" type=\"s\" name=\"subp"
        "ath\"/>\n      <arg direction=\"out\" type=\"o\" name=\"path\"/>\n"
        "    </method>\n    <method name=\"update\">\n      <arg direction="
        "\"in\" type=\"s\" name=\"path\"/>\n    </method>\n    <method name"
        "=\"sync\">\n      <arg direction=\"in\" type=\"s\" name=\"path\"/>"
        "\n    </method>\n    <method name=\"setDelayReleaseTime\">\n      "
        "<arg direction=\"in\" type=\"i\" name=\"time\"/>\n    </method>\n "
        "   <method name=\"delayReleaseTime\">\n      <arg direction=\"out\""
        " type=\"i\" name=\"time\"/>\n    </method>\n    <method name=\"ena"
        "bleVerboseLogging\"/>\n    <method name=\"disableVerboseLogging\"/"
        ">\n    <method name=\"setLogRules\">\n      <arg direction=\"in\" "
        "type=\"s\" name=\"rules\"/>\n    </method>\n    <method name=\"rem"
        "oveUserData\">\n      <arg direction=\"in\" type=\"u\" name=\"uid\""
        "/>\n    </method>\n    <method name=\"reload\"/>\n  </interface>\n",
        "acquireManager",
        "QDBusObjectPath",
        "",
        "appid",
        "name",
        "subpath",
        "acquireManagerV2",
        "uid",
        "delayReleaseTime",
        "disableVerboseLogging",
        "enableVerboseLogging",
        "reload",
        "removeUserData",
        "setDelayReleaseTime",
        "time",
        "setLogRules",
        "rules",
        "sync",
        "path",
        "update"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'acquireManager'
        QtMocHelpers::SlotData<QDBusObjectPath(const QString &, const QString &, const QString &)>(5, 7, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'acquireManagerV2'
        QtMocHelpers::SlotData<QDBusObjectPath(uint, const QString &, const QString &, const QString &)>(11, 7, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::UInt, 12 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'delayReleaseTime'
        QtMocHelpers::SlotData<int()>(13, 7, QMC::AccessPublic, QMetaType::Int),
        // Slot 'disableVerboseLogging'
        QtMocHelpers::SlotData<void()>(14, 7, QMC::AccessPublic, QMetaType::Void),
        // Slot 'enableVerboseLogging'
        QtMocHelpers::SlotData<void()>(15, 7, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reload'
        QtMocHelpers::SlotData<void()>(16, 7, QMC::AccessPublic, QMetaType::Void),
        // Slot 'removeUserData'
        QtMocHelpers::SlotData<void(uint)>(17, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 12 },
        }}),
        // Slot 'setDelayReleaseTime'
        QtMocHelpers::SlotData<void(int)>(18, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 19 },
        }}),
        // Slot 'setLogRules'
        QtMocHelpers::SlotData<void(const QString &)>(20, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Slot 'sync'
        QtMocHelpers::SlotData<void(const QString &)>(22, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Slot 'update'
        QtMocHelpers::SlotData<void(const QString &)>(24, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<DSGConfigAdaptor, qt_meta_tag_ZN16DSGConfigAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject DSGConfigAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DSGConfigAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DSGConfigAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16DSGConfigAdaptorE_t>.metaTypes,
    nullptr
} };

void DSGConfigAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DSGConfigAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QDBusObjectPath _r = _t->acquireManager((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusObjectPath*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDBusObjectPath _r = _t->acquireManagerV2((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QDBusObjectPath*>(_a[0]) = std::move(_r); }  break;
        case 2: { int _r = _t->delayReleaseTime();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->disableVerboseLogging(); break;
        case 4: _t->enableVerboseLogging(); break;
        case 5: _t->reload(); break;
        case 6: _t->removeUserData((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 7: _t->setDelayReleaseTime((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->setLogRules((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->sync((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->update((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *DSGConfigAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DSGConfigAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DSGConfigAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int DSGConfigAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}
QT_WARNING_POP
