/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.desktopspec.ConfigManager.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef CONFIGMANAGER_ADAPTOR_H
#define CONFIGMANAGER_ADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "dconfigserver.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.desktopspec.ConfigManager
 */
class DSGConfigAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.desktopspec.ConfigManager")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.desktopspec.ConfigManager\">\n"
"    <method name=\"acquireManagerV2\">\n"
"      <arg direction=\"in\" type=\"u\" name=\"uid\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"appid\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"subpath\"/>\n"
"      <arg direction=\"out\" type=\"o\" name=\"path\"/>\n"
"    </method>\n"
"    <method name=\"acquireManager\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"appid\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"subpath\"/>\n"
"      <arg direction=\"out\" type=\"o\" name=\"path\"/>\n"
"    </method>\n"
"    <method name=\"update\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"path\"/>\n"
"    </method>\n"
"    <method name=\"sync\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"path\"/>\n"
"    </method>\n"
"    <method name=\"setDelayReleaseTime\">\n"
"      <arg direction=\"in\" type=\"i\" name=\"time\"/>\n"
"    </method>\n"
"    <method name=\"delayReleaseTime\">\n"
"      <arg direction=\"out\" type=\"i\" name=\"time\"/>\n"
"    </method>\n"
"    <method name=\"enableVerboseLogging\"/>\n"
"    <method name=\"disableVerboseLogging\"/>\n"
"    <method name=\"setLogRules\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"rules\"/>\n"
"    </method>\n"
"    <method name=\"removeUserData\">\n"
"      <arg direction=\"in\" type=\"u\" name=\"uid\"/>\n"
"    </method>\n"
"    <method name=\"reload\"/>\n"
"  </interface>\n"
        "")
public:
    DSGConfigAdaptor(DSGConfigServer *parent);
    ~DSGConfigAdaptor() override;

    inline DSGConfigServer *parent() const
    { return static_cast<DSGConfigServer *>(QObject::parent()); }

public: // PROPERTIES
public Q_SLOTS: // METHODS
    QDBusObjectPath acquireManager(const QString &appid, const QString &name, const QString &subpath);
    QDBusObjectPath acquireManagerV2(uint uid, const QString &appid, const QString &name, const QString &subpath);
    int delayReleaseTime();
    void disableVerboseLogging();
    void enableVerboseLogging();
    void reload();
    void removeUserData(uint uid);
    void setDelayReleaseTime(int time);
    void setLogRules(const QString &rules);
    void sync(const QString &path);
    void update(const QString &path);
Q_SIGNALS: // SIGNALS
};

#endif
