/****************************************************************************
** Meta object code from reading C++ file 'manager_adaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "manager_adaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'manager_adaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN23DSGConfigManagerAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto DSGConfigManagerAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN23DSGConfigManagerAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DSGConfigManagerAdaptor",
        "D-Bus Interface",
        "org.desktopspec.ConfigManager.Manager",
        "D-Bus Introspection",
        "  <interface name=\"org.desktopspec.ConfigManager.Manager\">\n    "
        "<property access=\"read\" type=\"s\" name=\"version\"/>\n    <prop"
        "erty access=\"read\" type=\"as\" name=\"keyList\"/>\n    <method n"
        "ame=\"value\">\n      <arg direction=\"in\" type=\"s\" name=\"key\""
        "/>\n      <arg direction=\"out\" type=\"v\" name=\"value\"/>\n    "
        "</method>\n    <method name=\"isDefaultValue\">\n      <arg direct"
        "ion=\"in\" type=\"s\" name=\"key\"/>\n      <arg direction=\"out\""
        " type=\"b\" name=\"isDefaultValue\"/>\n    </method>\n    <method "
        "name=\"setValue\">\n      <arg direction=\"in\" type=\"s\" name=\""
        "key\"/>\n      <arg direction=\"in\" type=\"v\" name=\"value\"/>\n"
        "    </method>\n    <method name=\"reset\">\n      <arg direction=\""
        "in\" type=\"s\" name=\"key\"/>\n    </method>\n    <method name=\""
        "name\">\n      <arg direction=\"in\" type=\"s\" name=\"key\"/>\n  "
        "    <arg direction=\"in\" type=\"s\" name=\"language\"/>\n      <a"
        "rg direction=\"out\" type=\"s\" name=\"name\"/>\n    </method>\n  "
        "  <method name=\"description\">\n      <arg direction=\"in\" type="
        "\"s\" name=\"key\"/>\n      <arg direction=\"in\" type=\"s\" name="
        "\"language\"/>\n      <arg direction=\"out\" type=\"s\" name=\"des"
        "cription\"/>\n    </method>\n    <method name=\"visibility\">\n   "
        "   <arg direction=\"in\" type=\"s\" name=\"key\"/>\n      <arg dir"
        "ection=\"out\" type=\"s\" name=\"visibility\"/>\n    </method>\n  "
        "  <method name=\"permissions\">\n      <arg direction=\"in\" type="
        "\"s\" name=\"key\"/>\n      <arg direction=\"out\" type=\"s\" name"
        "=\"permissions\"/>\n    </method>\n    <method name=\"flags\">\n  "
        "    <arg direction=\"in\" type=\"s\" name=\"key\"/>\n      <arg di"
        "rection=\"out\" type=\"i\" name=\"flags\"/>\n    </method>\n    <m"
        "ethod name=\"release\"/>\n    <signal name=\"valueChanged\">\n    "
        "  <arg direction=\"out\" type=\"s\" name=\"key\"/>\n    </signal>\n"
        "  </interface>\n",
        "valueChanged",
        "",
        "key",
        "description",
        "language",
        "flags",
        "isDefaultValue",
        "name",
        "permissions",
        "release",
        "reset",
        "setValue",
        "QDBusVariant",
        "value",
        "visibility",
        "keyList",
        "version"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'valueChanged'
        QtMocHelpers::SignalData<void(const QString &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'description'
        QtMocHelpers::SlotData<QString(const QString &, const QString &)>(8, 6, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 7 }, { QMetaType::QString, 9 },
        }}),
        // Slot 'flags'
        QtMocHelpers::SlotData<int(const QString &)>(10, 6, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'isDefaultValue'
        QtMocHelpers::SlotData<bool(const QString &)>(11, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'name'
        QtMocHelpers::SlotData<QString(const QString &, const QString &)>(12, 6, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 7 }, { QMetaType::QString, 9 },
        }}),
        // Slot 'permissions'
        QtMocHelpers::SlotData<QString(const QString &)>(13, 6, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'release'
        QtMocHelpers::SlotData<void()>(14, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reset'
        QtMocHelpers::SlotData<void(const QString &)>(15, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'setValue'
        QtMocHelpers::SlotData<void(const QString &, const QDBusVariant &)>(16, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { 0x80000000 | 17, 18 },
        }}),
        // Slot 'value'
        QtMocHelpers::SlotData<QDBusVariant(const QString &)>(18, 6, QMC::AccessPublic, 0x80000000 | 17, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'visibility'
        QtMocHelpers::SlotData<QString(const QString &)>(19, 6, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'keyList'
        QtMocHelpers::PropertyData<QStringList>(20, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'version'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<DSGConfigManagerAdaptor, qt_meta_tag_ZN23DSGConfigManagerAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject DSGConfigManagerAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23DSGConfigManagerAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23DSGConfigManagerAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23DSGConfigManagerAdaptorE_t>.metaTypes,
    nullptr
} };

void DSGConfigManagerAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DSGConfigManagerAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->valueChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: { QString _r = _t->description((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 2: { int _r = _t->flags((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->isDefaultValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { QString _r = _t->name((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 5: { QString _r = _t->permissions((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->release(); break;
        case 7: _t->reset((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->setValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusVariant>>(_a[2]))); break;
        case 9: { QDBusVariant _r = _t->value((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusVariant*>(_a[0]) = std::move(_r); }  break;
        case 10: { QString _r = _t->visibility((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DSGConfigManagerAdaptor::*)(const QString & )>(_a, &DSGConfigManagerAdaptor::valueChanged, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QStringList*>(_v) = _t->keyList(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->version(); break;
        default: break;
        }
    }
}

const QMetaObject *DSGConfigManagerAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DSGConfigManagerAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23DSGConfigManagerAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int DSGConfigManagerAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void DSGConfigManagerAdaptor::valueChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
