/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.desktopspec.ConfigManager.Manager.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "manager_adaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class DSGConfigManagerAdaptor
 */

DSGConfigManagerAdaptor::DSGConfigManagerAdaptor(DSGConfigConn *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

DSGConfigManagerAdaptor::~DSGConfigManagerAdaptor()
{
    // destructor
}

QStringList DSGConfigManagerAdaptor::keyList() const
{
    // get the value of property keyList
    return qvariant_cast< QStringList >(parent()->property("keyList"));
}

QString DSGConfigManagerAdaptor::version() const
{
    // get the value of property version
    return qvariant_cast< QString >(parent()->property("version"));
}

QString DSGConfigManagerAdaptor::description(const QString &key, const QString &language)
{
    // handle method call org.desktopspec.ConfigManager.Manager.description
    return parent()->description(key, language);
}

int DSGConfigManagerAdaptor::flags(const QString &key)
{
    // handle method call org.desktopspec.ConfigManager.Manager.flags
    return parent()->flags(key);
}

bool DSGConfigManagerAdaptor::isDefaultValue(const QString &key)
{
    // handle method call org.desktopspec.ConfigManager.Manager.isDefaultValue
    return parent()->isDefaultValue(key);
}

QString DSGConfigManagerAdaptor::name(const QString &key, const QString &language)
{
    // handle method call org.desktopspec.ConfigManager.Manager.name
    return parent()->name(key, language);
}

QString DSGConfigManagerAdaptor::permissions(const QString &key)
{
    // handle method call org.desktopspec.ConfigManager.Manager.permissions
    return parent()->permissions(key);
}

void DSGConfigManagerAdaptor::release()
{
    // handle method call org.desktopspec.ConfigManager.Manager.release
    parent()->release();
}

void DSGConfigManagerAdaptor::reset(const QString &key)
{
    // handle method call org.desktopspec.ConfigManager.Manager.reset
    parent()->reset(key);
}

void DSGConfigManagerAdaptor::setValue(const QString &key, const QDBusVariant &value)
{
    // handle method call org.desktopspec.ConfigManager.Manager.setValue
    parent()->setValue(key, value);
}

QDBusVariant DSGConfigManagerAdaptor::value(const QString &key)
{
    // handle method call org.desktopspec.ConfigManager.Manager.value
    return parent()->value(key);
}

QString DSGConfigManagerAdaptor::visibility(const QString &key)
{
    // handle method call org.desktopspec.ConfigManager.Manager.visibility
    return parent()->visibility(key);
}


#include "moc_manager_adaptor.cpp"
